from sidekick import lazy, placeholder as this
from .environment import EnvironmentConf
from ..descriptors import env


class DatabaseConf(EnvironmentConf):
    """
    Configure the database.

    See also: https://docs.djangoproject.com/en/2.0/ref/settings/#databases
    """

    DATABASE_DEFAULT = env('sqlite:///local/db/db.sqlite3',
                           type='db_url',
                           name='DJANGO_DB_URL')

    DATABASES = lazy(lambda self: {
        'default': dict(TEST={}, **self.DATABASE_DEFAULT),
    })

    # Derived inspections
    USING_SQLITE = lazy(this.is_using_db('sqlite'))
    USING_POSTGRESQL = lazy(this.is_using_db('postgresql'))
    USING_MYSQL = lazy(this.is_using_db('mysql'))

    def is_using_db(self, db, which='default'):
        return db in self.DATABASES[which]['ENGINE']



from ..base import Conf
from ..descriptors import env, env_property


class EnvironmentConf(Conf):
    """
    Base class for Django configurations. Users should use DjangoConf
    """

    env_prefix = 'DJANGO_'

    #: Django base environment. We suggest distinguishing between 'local',
    #: 'test' and 'production'.
    ENVIRONMENT = env('local')

    #: By default, debug is enabled only on 'local' and 'test' environments.
    @env_property(type=bool)
    def DEBUG(self, value):  # noqa: N802
        if value is None:
            return self.ENVIRONMENT == 'local'
        return value

    @env_property
    def WSGI_APPLICATION(self, value):  # noqa: N802
        if value is None:
            return self.get_django_project_path() + '.wsgi.application'
        return value

    #: It is often convenient to enable/disable Django ability to serve static
    #: files.
    SERVE_STATIC_FILES = env(True)

    #: Site admin url, if Django admin app is installed.
    SITE_ID = 1

    #: Authentication
    AUTH_USER_MODEL = 'auth.User'



import sys;

from setuptools import setup, find_packages

sys.path.append('src')

setup(
    name='django-boogie',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    setup_requires='setuptools >= 30.3',
)



from .environment import EnvironmentConf
from ..descriptors import env, env_property


class UrlsConf(EnvironmentConf):
    """
    Configure urls for your Django project.
    """

    ADMIN_URL = env('admin/')
    LOGIN_URL = env('/login/')
    LOGOUT_URL = env('/logout/')
    STATIC_URL = env('/static/')
    MEDIA_URL = env('/media/')

    @env_property
    def ROOT_URLCONF(self, value):  # noqa: N802
        if value is None:
            return self.get_django_project_path() + '.urls'
        return value

    APPEND_SLASH = True



from .environment import EnvironmentConf
from ..tools import secret_hash


class SecurityConf(EnvironmentConf):
    """
    Security options.
    """

    def finalize(self, settings):
        settings = super().finalize(settings)
        if not settings.get('SECRET_KEY'):
            settings['SECRET_KEY'] = secret_hash(settings)
        return settings

    #: WARNING: keep the secret key used in production secret! We generate a
    #: secret from a hash of the current settings during the .finalize() phase.
    #: this is ok for local development, but may be insecure/inconvenient for
    def get_secret_key(self):  # noqa: N802
        value = self.env.str('DJANGO_SECRET_KEY', default=None)
        if not value:
            if self.ENVIRONMENT in ('local', 'test'):
                return self.ENVIRONMENT
            else:
                return None
        return value

    # Password validation
    # https://docs.djangoproject.com/en/2.0/ref/settings/#auth-password-validators
    AUTH_PASSWORD_VALIDATORS = [
        {
            'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
        },
        {
            'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
        },
        {
            'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
        },
        {
            'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
        },
    ]

    def get_allowed_hosts(self):
        return self.env('DJANGO_ALLOWED_HOSTS', type=list, default=['localhost'])

    def get_password_hashers(self):
        if self.ENVIRONMENT == 'testing':
            return [
                'django.contrib.auth.hashers.MD5PasswordHasher',
            ]
        return [
            'django.contrib.auth.hashers.Argon2PasswordHasher',
            'django.contrib.auth.hashers.PBKDF2PasswordHasher',
            'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
            'django.contrib.auth.hashers.BCryptSHA256PasswordHasher',
            'django.contrib.auth.hashers.BCryptPasswordHasher',
        ]
